local oldSiloUpdate = BunkerSilo.update;
BunkerSilo.update = function(self,dt)
    if self.fermentingTimeNEW == nil then
        self.fermentingTimeNEW = 0;
    end;
    if self.state == BunkerSilo.STATE_CLOSED then
        if self.silagePlaneId ~= nil and self.silagePlaneMoveAlpha < self.silagePlaneMoveAlphaMax then
            self.silagePlaneMoveAlpha = math.min(self.silagePlaneMoveAlpha + dt / self.silagePlaneMoveDuration, self.silagePlaneMoveAlphaMax)
            local x, _, z = getTranslation(self.silagePlaneId)
            setTranslation(self.silagePlaneId, x, self.silagePlaneMinY + self.silagePlaneMoveAlpha * (self.silagePlaneMaxY - self.silagePlaneMinY), z)
        end
        self.fermentingTime = 0;
        if self.fermentingTimeNEW < self.fermentingDuration then
            self.fermentingTimeNEW = self.fermentingTimeNEW + dt * 0.001 * g_currentMission.missionStats.timeScale
        else
            self.fermentingTimeNEW = self.fermentingDuration;
        end
        if self.fermentingTimeNEW >= self.fermentingDuration then
            if self:getCanInteract(true) then
                g_currentMission:addHelpButtonText("Silo aufdecken", InputBinding.ACTIVATE_OBJECT)
                if InputBinding.hasEvent(InputBinding.ACTIVATE_OBJECT) then
                    self:setState(BunkerSilo.STATE_DRAIN)
                    if g_server == nil then
                        g_client:getServerConnection():sendEvent(BunkerSiloOpenEvent:new(self));
                    end;
                end;
            end;
        end
    elseif self.state == BunkerSilo.STATE_DRAIN then
        if self:getCanInteract(true) then
            g_currentMission:addHelpButtonText(g_i18n:getText("blanket_silo"), InputBinding.ACTIVATE_OBJECT)
            if InputBinding.hasEvent(InputBinding.ACTIVATE_OBJECT) then
                self:setState(BunkerSilo.STATE_CLOSED)
                if g_server == nil then
                    g_client:getServerConnection():sendEvent(BunkerSiloCloseNewEvent:new(self));
                end;
            end;
        end;
        local colorScale = 0.8;
        local colorScaleR = 210/255*colorScale;
        local colorScaleG = 180/255*colorScale;
        local colorScaleB = 141/255*colorScale;
        for _,movingPlane in pairs(self.movingPlanes) do
            setShaderParameter(movingPlane.nodeId, "colorScale", colorScaleR, colorScaleG, colorScaleB, 0, false);
        end;
    end
    if self:getCanInteract(true) then
        if self.showMovingPlaneFullWarningTimer > 0 then
            g_currentMission:addWarning(g_i18n:getText("Here_is_no_space"), 0.018, 0.033);
        end;
        if self.state == BunkerSilo.STATE_FILL then
            g_currentMission:addExtraPrintText(g_i18n:getText("fill_level").." "..math.floor(self.fillLevel).." ("..math.floor(100*self.fillLevel/self.capacity).."%)");
            local compactedPercentage = 0;
            if self.fillLevel > 0 then
                compactedPercentage = math.min(self.compactedFillLevel/self.fillLevel, 1);
            end;
            g_currentMission:addExtraPrintText(g_i18n:getText("compacting").." "..math.floor(100*compactedPercentage).."%");
        elseif self.state == BunkerSilo.STATE_CLOSED then
            g_currentMission:addExtraPrintText(g_i18n:getText("fermenting").." "..math.floor(100*self.fermentingTimeNEW/self.fermentingDuration).."%");
        elseif self.state == BunkerSilo.STATE_DRAIN then
            g_currentMission:addExtraPrintText(g_i18n:getText("fill_level").." "..math.floor(self.fillLevel).." ("..math.floor(100*self.fillLevel/self.capacity).."%)");
        end;
    end;
    oldSiloUpdate(self,dt)
end

local oldCanInteract = BunkerSilo.getCanInteract;
BunkerSilo.getCanInteract = function(self,add)
    if(add == nil) then
        return false;
    else
        return oldCanInteract(self);
    end;
end;

BunkerSiloActivatable.getIsActivatable = function(self)
  if self.bunkerSilo:getCanInteract(true) and self.bunkerSilo:getCanCloseSilo() then
    self:updateActivateText()
    return true
  end
  return false
end

local oldWriteStream = BunkerSilo.writeStream;
BunkerSilo.writeStream = function(self,streamId,connection)
    BunkerSilo.superClass().writeStream(self, streamId, connection);
    if not connection:getIsServer() then
        streamWriteUIntN(streamId, self.state, 2);
        for i=1, table.getn(self.movingPlanes) do
            local movingPlane = self.movingPlanes[i];
            streamWriteFloat32(streamId, movingPlane.fillLevel);
        end;
        streamWriteFloat32(streamId, self.compactedFillLevel);
        streamWriteFloat32(streamId, self.fermentingTimeNEW);
        streamWriteFloat32(streamId, self.silagePlaneMoveAlphaMax);
        streamWriteFloat32(streamId, self.silagePlaneMoveAlpha);
    end;
end;

BunkerSilo.readStream = function(self,streamId,connection)
    BunkerSilo:superClass().readStream(self, streamId, connection);
    if connection:getIsServer() then
        local state = streamReadUIntN(streamId, 2);
        self:setState(state);
        for i=1, table.getn(self.movingPlanes) do
            local movingPlane = self.movingPlanes[i];
            local fillLevel = streamReadFloat32(streamId);
            self:setMovingPlaneFillLevel(movingPlane, fillLevel, false);
        end;
        self.compactedFillLevel = streamReadFloat32(streamId);
        self.fermentingTimeNEW = streamReadFloat32(streamId);
        self.silagePlaneMoveAlphaMax = streamReadFloat32(streamId);
        self.silagePlaneMoveAlpha = streamReadFloat32(streamId);
        if self.state ~= BunkerSilo.STATE_CLOSED then
            self.silagePlaneMoveAlpha = 0;
        end;
        if self.silagePlaneId ~= nil then
            local x,_,z = getTranslation(self.silagePlaneId);
            setTranslation(self.silagePlaneId, x, self.silagePlaneMinY + self.silagePlaneMoveAlpha*(self.silagePlaneMaxY- self.silagePlaneMinY), z);
        end;
    end;
end;

BunkerSilo.getSaveAttributesAndNodes = function(self, nodeIdent)
    local attributes = "state=\"" .. self.state .. "\" compactedFillLevel=\"" .. self.compactedFillLevel .. "\" fermentingTime=\"" .. self.fermentingTimeNEW .. "\" silagePlaneMoveAlpha=\"" .. self.silagePlaneMoveAlpha .. "\" silagePlaneMoveAlphaMax=\"" .. self.silagePlaneMoveAlphaMax .. "\""
    local nodes = ""
    for i = 1, table.getn(self.movingPlanes) do
        if i > 1 then
            nodes = nodes .. "\n"
        end
        local movingPlane = self.movingPlanes[i]
        nodes = nodes .. nodeIdent .. "<movingPlane" .. i .. " fillLevel=\"" .. movingPlane.fillLevel .. "\" />"
    end
    return attributes, nodes
end;

BunkerSilo.loadFromAttributesAndNodes = function(self, xmlFile, key)
    local state = getXMLInt(xmlFile, key .. "#state")
    if state ~= nil and state >= 0 and state < BunkerSilo.NUM_STATES then
        self:setState(state)
    end
    for i = 1, table.getn(self.movingPlanes) do
        local planeKey = key .. string.format(".movingPlane%d", i)
        if not hasXMLProperty(xmlFile, planeKey) then
            break
        end
        local fillLevel = getXMLFloat(xmlFile, planeKey .. "#fillLevel")
        if fillLevel ~= nil then
            self:setMovingPlaneFillLevel(self.movingPlanes[i], fillLevel, false)
        end
    end
    local compactedFillLevel = getXMLFloat(xmlFile, key .. "#compactedFillLevel")
    if compactedFillLevel ~= nil then
        self.compactedFillLevel = Utils.clamp(compactedFillLevel, 0, self.fillLevel)
    end
    if self.fermentingTimeNEW == nil then
        self.fermentingTimeNEW = 0;
    end;
    local fermentingTime = getXMLFloat(xmlFile, key .. "#fermentingTime")
    if fermentingTime ~= nil then
        self.fermentingTimeNEW = Utils.clamp(fermentingTime, 0, self.fermentingDuration)
    end
    self:calculateSilagePlaneMoveAlphaMax()
    local silagePlaneMoveAlphaMax = getXMLFloat(xmlFile, key .. "#silagePlaneMoveAlphaMax")
    if silagePlaneMoveAlphaMax ~= nil then
        self.silagePlaneMoveAlphaMax = math.max(silagePlaneMoveAlphaMax, self.silagePlaneMoveAlphaMax)
    end
    local silagePlaneMoveAlpha = getXMLFloat(xmlFile, key .. "#silagePlaneMoveAlpha")
    if silagePlaneMoveAlpha ~= nil then
        self.silagePlaneMoveAlpha = Utils.clamp(silagePlaneMoveAlpha, 0, self.silagePlaneMoveAlphaMax)
        if self.state ~= BunkerSilo.STATE_CLOSED then
            self.silagePlaneMoveAlpha = 0
        end
        if self.silagePlaneId ~= nil then
            local x, _, z = getTranslation(self.silagePlaneId)
            setTranslation(self.silagePlaneId, x, self.silagePlaneMinY + self.silagePlaneMoveAlpha * (self.silagePlaneMaxY - self.silagePlaneMinY), z)
        end
    end
    return true
end

BunkerSiloOpenEvent = {};
BunkerSiloOpenEvent_mt = Class(BunkerSiloOpenEvent, Event);

InitEventClass(BunkerSiloOpenEvent, "BunkerSiloOpenEvent");

function BunkerSiloOpenEvent:emptyNew()
    local self = Event:new(BunkerSiloOpenEvent_mt);
    return self;
end;

function BunkerSiloOpenEvent:new(bunkerSilo)
    local self = BunkerSiloOpenEvent:emptyNew()
    self.bunkerSilo = bunkerSilo;
    return self;
end;

function BunkerSiloOpenEvent:readStream(streamId, connection)
    if not connection:getIsServer() then
        self.bunkerSilo = networkGetObject(streamReadInt32(streamId));
    end;
    self:run(connection);
end;

function BunkerSiloOpenEvent:writeStream(streamId, connection)
    if connection:getIsServer() then
        streamWriteInt32(streamId, networkGetObjectId(self.bunkerSilo));
    end;
end;

function BunkerSiloOpenEvent:run(connection)
    if not connection:getIsServer() then
        self.bunkerSilo:setState(BunkerSilo.STATE_DRAIN);
    end;
end;

BunkerSiloCloseNewEvent = {};
BunkerSiloCloseNewEvent_mt = Class(BunkerSiloCloseNewEvent, Event);

InitEventClass(BunkerSiloCloseNewEvent, "BunkerSiloCloseNewEvent");

function BunkerSiloCloseNewEvent:emptyNew()
    local self = Event:new(BunkerSiloCloseNewEvent_mt);
    return self;
end;

function BunkerSiloCloseNewEvent:new(bunkerSilo)
    local self = BunkerSiloCloseNewEvent:emptyNew()
    self.bunkerSilo = bunkerSilo;
    return self;
end;

function BunkerSiloCloseNewEvent:readStream(streamId, connection)
    if not connection:getIsServer() then
        self.bunkerSilo = networkGetObject(streamReadInt32(streamId));
    end;
    self:run(connection);
end;

function BunkerSiloCloseNewEvent:writeStream(streamId, connection)
    if connection:getIsServer() then
        streamWriteInt32(streamId, networkGetObjectId(self.bunkerSilo));
    end;
end;

function BunkerSiloCloseNewEvent:run(connection)
    if not connection:getIsServer() then
        self.bunkerSilo:setState(BunkerSilo.STATE_CLOSED);
        local colorScale = 0.8;
        local colorScaleR = 210/255*colorScale;
        local colorScaleG = 180/255*colorScale;
        local colorScaleB = 141/255*colorScale;
        for _,movingPlane in pairs(self.bunkerSilo.movingPlanes) do
            setShaderParameter(movingPlane.nodeId, "colorScale", colorScaleR, colorScaleG, colorScaleB, 0, false);
        end;
    end;
end;